@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2015 - 2020, Stephan Mueller <smueller@chronox.de>
+ * Copyright (C) 2015 - 2021, Stephan Mueller <smueller@chronox.de>
  *
  * License: see LICENSE file in root directory
  *
@@ -95,7 +95,7 @@ static void cp_hash_fini_test(struct cp_test *test)
 	free(test->u.hash.scratchpad);
 }
 
-static unsigned int cp_hash_test(struct cp_test *test)
+static size_t cp_hash_test(struct cp_test *test)
 {
 	kcapi_md_digest(test->u.hash.handle,
 			test->u.hash.scratchpad,
