@@ -1,6 +1,6 @@
 #!/bin/bash
 #
-# Copyright (C) 2017 - 2020, Stephan Mueller <smueller@chronox.de>
+# Copyright (C) 2017 - 2021, Stephan Mueller <smueller@chronox.de>
 #
 # License: see LICENSE file in root directory
 #
@@ -320,12 +320,22 @@ function run_kat() {
 	result="$1"; shift
 	truncate="$1"; shift
 
+	# The following tests do not work on eudyptula
+	# See below for the offending invocation
+	if uname -n | grep -q eudyptula
+	then
+		echo_deact "Hasher test deactivated"
+		return
+	fi
+
 	truncate_opt=''
 	[ -z "$truncate" ] || truncate_opt="-t $truncate"
 
 	expand_string "$data" >"$ANOTHER"
 	echo "${result#0x}  $ANOTHER" >"$CHKFILE"
 
+	# The -k requires a file descriptor which cannot be created in the
+	# eudyptula Hypervsior test environment
 	run_hasher $hasher -q \
 		-k <(expand_string "$key") -c "$CHKFILE" $truncate_opt
 	if [ $? -ne 0 ]
